# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"indexFlow": "00_core.ipynb",
         "queryFlow": "00_core.ipynb",
         "slugify": "01_loader.ipynb",
         "get_image_files": "01_loader.ipynb",
         "device": "03_encoder.ipynb",
         "archive_loader": "01_loader.ipynb",
         "db_loader": "01_loader.ipynb",
         "treemap_loader": "01_loader.ipynb",
         "make_dataset": "02_crafter.ipynb",
         "pil_loader": "02_crafter.ipynb",
         "DatasetImagePaths": "02_crafter.ipynb",
         "clip_transform": "02_crafter.ipynb",
         "crafter": "02_crafter.ipynb",
         "image_encoder": "03_encoder.ipynb",
         "text_encoder": "03_encoder.ipynb",
         "join_all": "04_indexer.ipynb",
         "build_treemap": "04_indexer.ipynb",
         "save_archives": "04_indexer.ipynb",
         "ranker": "05_ranker.ipynb",
         "nns_to_files": "05_ranker.ipynb",
         "top_bottom": "06_fileutils.ipynb",
         "copy_topn_images": "06_fileutils.ipynb",
         "app": "07_cli.ipynb",
         "search_folder": "07_cli.ipynb",
         "__main__": "07_cli.ipynb",
         "get_image": "08_gui.ipynb",
         "get_grid": "08_gui.ipynb",
         "update_tabs": "08_gui.ipynb",
         "page_update": "08_gui.ipynb",
         "appPage": "08_gui.ipynb"}

modules = ["core.py",
           "loader.py",
           "crafter.py",
           "encoder.py",
           "indexer.py",
           "ranker.py",
           "utils.py",
           "cli.py",
           "gui.py"]

doc_url = "https://deepfates.github.io/memery/"

git_url = "https://github.com/deepfates/memery/tree/main/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: 08_gui.ipynb (unless otherwise specified).

__all__ = ['get_image', 'get_grid', 'update_tabs', 'page_update', 'appPage']

# Cell
import ipywidgets as widgets

from .core import queryFlow
from pathlib import Path
from IPython.display import clear_output



# Cell
def get_image(file_loc):
    filepath = Path(file_loc)
    file = open(filepath, 'rb')
    image = widgets.Image(value=file.read(),width=200)
    return(image)

# Cell
def get_grid(filepaths, n=4):
    imgs = [get_image(f) for f in filepaths[:n]]
    grid = widgets.GridBox(imgs, layout=widgets.Layout(grid_template_columns="repeat(auto-fit, 200px)"))
    return(grid)

# Cell
def update_tabs(path, query, n_images, searches, tabs, inputs):
    if query.value:
        ranked = queryFlow(path.value, query.value)

        stem = Path(path.value).stem
        slug = f"{stem}:{str(query.value)}"
        searches[f'{slug}'] = ranked

    tabs.children = [get_grid(v, n=n_images.value) for v in searches.values()]
    tabs.selected_index = len(searches)-1
    for i, k in enumerate(searches.keys()):
        tabs.set_title(i, k)

#     return(True)

# Cell
def page_update(b):
    update_tabs(path, query, n_images, searches, tabs, inputs)


# Cell
class appPage():

    def __init__(self):
        self.inputs_layout =  widgets.Layout(max_width='80%')

        self.path = widgets.Text(placeholder='path/to/image/folder', value='images/', layout=self.inputs_layout)
        self.query = widgets.Text(placeholder='a funny dog meme', value='a funny dog meme', layout=self.inputs_layout)

        self.n_images = widgets.IntSlider(description='#', value=4, layout=self.inputs_layout)
        self.go = widgets.Button(description="Search", layout=self.inputs_layout)

        self.all_inputs_layout =  widgets.Layout(max_width='100%', min_height='40vh', flex_flow='row wrap', align_content='flex-start')

        self.inputs = widgets.Box([self.path, self.query, self.n_images, self.go], layout=self.all_inputs_layout)
        self.tabs = widgets.Tab()
        self.page = widgets.AppLayout(left_sidebar=self.inputs, center=self.tabs)

        self.searches = {}
        self.go.on_click(self.page_update)

        display(self.page)

    def page_update(self, b):
        update_tabs(self.path, self.query, self.n_images, self.searches, self.tabs, self.inputs)




# AUTOGENERATED! DO NOT EDIT! File to edit: 00_core.ipynb (unless otherwise specified).

__all__ = ['indexFlow', 'queryFlow']

# Cell
import torch
from pathlib import Path
from .loader import get_image_files, archive_loader, db_loader, treemap_loader
from .crafter import crafter
from .encoder import image_encoder, text_encoder
from .indexer import join_all, build_treemap, save_archives
from .ranker import ranker, nns_to_files

# Cell
def indexFlow(path):
    root = Path(path)
    device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

    filepaths = get_image_files(root)
    archive_db = {}

    archive_db, new_files = archive_loader(filepaths, root, device)
    print(f"Loaded {len(archive_db)} encodings")
    print(f"Encoding {len(new_files)} new images")
    crafted_files = crafter(new_files, device)
    new_embeddings = image_encoder(crafted_files, device)

    db = join_all(archive_db, new_files, new_embeddings)
    print("Building treemap")
    t = build_treemap(db)

    print(f"Saving {len(db)}images")
    save_paths = save_archives(root, t, db)
    print("Done")
    return(save_paths)

# Cell
def queryFlow(path, query):
    root = Path(path)
    device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

    dbpath = root/'memery.pt'
    db = db_loader(dbpath, device)
    treepath = root/'memery.ann'
    treemap = treemap_loader(treepath)

    if treemap == None or db == {}:
        dbpath, treepath = indexFlow(root)
        treemap = treemap_loader(Path(treepath))
        db = db_loader(dbpath, device)

    print(f"Searching {len(db)} images")
    query_vec = text_encoder(query, device)
    indexes = ranker(query_vec, treemap)
    ranked_files = nns_to_files(db, indexes)
    return(ranked_files)




# AUTOGENERATED! DO NOT EDIT! File to edit: 05_ranker.ipynb (unless otherwise specified).

__all__ = ['ranker', 'nns_to_files']

# Cell
def ranker(query_vec, treemap):
    nn_indexes = treemap.get_nns_by_vector(query_vec[0], treemap.get_n_items())
    return(nn_indexes)

# Cell
def nns_to_files(db, indexes):
    return([[v['fpath'] for k,v in db.items() if v['index'] == ind][0] for ind in indexes])


# AUTOGENERATED! DO NOT EDIT! File to edit: 07_cli.ipynb (unless otherwise specified).

__all__ = ['app', 'search_folder', '__main__']

# Cell
import typer
from .core import queryFlow


# Cell
app = typer.Typer()

# Cell
@app.command()
def search_folder(path: str, query: str, n: int = 10):
    ranked = queryFlow(path, query)
    print(ranked[:n])
#     return(ranked)


# Cell
def __main__():
    app()